import os
import yfinance as yf

import warnings
warnings.filterwarnings("ignore")

class Yahooer:
    def __init__(self) -> None:
        self.market = "us"

        self.home = "%s" % (os.getenv('HOME'))

        self.cacheDir = "%s/%s" % (self.home, ".cache/yf")
        yf.set_tz_cache_location(self.cacheDir)

        self.csvDir = "%s/project/data/yf" % (self.home)

    def getK(self, _code):
        msft = yf.Ticker(_code)
        hist = msft.history(period="max")
        if len(hist) <= 10:
            return hist
        #print(hist.head())
        hist = hist.drop('Dividends', axis=1, errors="ignore")
        hist = hist.drop('Stock Splits', axis=1, errors="ignore")
        hist = hist.drop('Capital Gains', axis=1, errors="ignore")
        hist = hist.reset_index()
        hist['Date'] = hist['Date'].dt.date
        hist = hist.set_index('Date')
        hist = hist.dropna()
        csvPath = "%s/%s.csv" % (self.csvDir, _code)
        #print("--------", _code, csvPath)
        hist.to_csv(csvPath)
        return hist

    def getKByList(self):
        # Define the file path
        list_path = "%s/project/data/list/us.txt" % (self.home)

        try:
            with open(list_path, 'r') as file:
                for line in file:
                    code = line.strip()
                    print(code)
                    data = self.getK(code)
        except FileNotFoundError:
            print(f"File '{list_path}' not found.")
        except Exception as e:
            print(f"An error occurred: {e}")

if __name__=="__main__":
    y = Yahooer()
    y.getK('IONQ')
    # ya.getKByList()



#!/usr/bin/env python
# -*- encoding: utf8 -*-

from jing.yahooer import Yahooer
from jing.aker import AKER

import pandas as pd

import warnings
warnings.filterwarnings("ignore")

pd.set_option('display.max_columns', None)

class Z:
    def __init__(self, _market="us") -> None:
        self.market = _market
        if self.market == 'us':
            self.inst = Yahooer()
        elif self.market == 'hk' or self.market == 'cn':
            self.inst = AKER(self.market)
        else:
            self.inst = Yahooer()

    def download(self, _code=""):
        self.code = _code
        if len(_code) > 0:
            self.inst.getK(self.code)
        else:
            self.inst.getKByList()

if __name__=="__main__":
    z = Z()
    z.download("PLTR")
    z2 = Z(_market='cn')
    z2.download('600863')



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import pandas as pd
from jing.data_center import DataCenter

class Tech:
    def __init__(self, _df) -> None:
        self.df = _df

        self.df.set_index("Date", inplace=True)
        self.df = self.df.sort_index(ascending=True)


    def calc(self):
        price = self.df['Close']

        # sma
        self.df['ma20']  = calc_sma(price, 20)
        self.df['ma50']  = calc_sma(price, 50)
        self.df['ma200'] = calc_sma(price, 200)

        # vma
        vol = self.df['Volume']
        self.df['vma50'] = calc_sma(vol, 50)

        # macd
        diff, dea, macd = calc_macd(price)
        self.df['macd'] = macd
        self.df['diff'] = diff
        self.df['dea'] = dea

        return self.df.dropna()

def calc_sma(_sc, _n):
    return _sc.rolling(_n).mean()

def calc_macd(_sc):
    # Get the 26-day EMA of the closing price
    k = _sc.ewm(span=12, adjust=False, min_periods=12).mean()
    # Get the 12-day EMA of the closing price
    d = _sc.ewm(span=26, adjust=False, min_periods=26).mean()

    # Subtract the 26-day EMA from the 12-Day EMA to get the MACD
    macd = k - d

    # Get the 9-Day EMA of the MACD for the Trigger line
    macd_s = macd.ewm(span=9, adjust=False, min_periods=9).mean()

    # Calculate the difference between the MACD - Trigger for the Convergence/Divergence value
    macd_h = 2 * (macd - macd_s)

    return macd, macd_s, macd_h # diff, dea, macd

if __name__ == "__main__":
    data_center = DataCenter()
    df = data_center.one("IONQ")
    pd.set_option('display.max_columns', None)
    df = Tech(df).calc()
    print(df)



from binance.client import Client
import pandas as pd
import os
from datetime import date

import warnings
warnings.filterwarnings("ignore")

class BN:
    def __init__(self) -> None:
        self.home = "%s" % (os.getenv('HOME'))
        self.csvDir = "%s/project/data/bn" % (self.home)
        self.market = "bn"
        api_key = os.getenv('BINANCE_API_KEY')
        api_secret = os.getenv('BINANCE_API_SECRET')
        self.client = Client(api_key, api_secret)
        self.start_date = '2017-01-01'
        today = date.today()
        self.end_date = today.strftime("%Y-%m-%d")
        self.interval = Client.KLINE_INTERVAL_1DAY
        # self.interval = Client.KLINE_INTERVAL_4HOUR


    def getK(self, _code):
        klines = self.client.get_historical_klines(_code, Client.KLINE_INTERVAL_1DAY, self.start_date, self.end_date)
        df = pd.DataFrame(klines, columns=['timestamp', 'Open', 'High', 'Low', 'Close', 'Volume', 'close_time', 'quote_asset_volume', 'number_of_trades', 'taker_buy_base_asset_volume', 'taker_buy_quote_asset_volume', 'ignore'])
        df['Date'] = pd.to_datetime(df['timestamp'], unit='ms')
        hist = df.drop(['timestamp', 'close_time', 'quote_asset_volume', 'number_of_trades', 'taker_buy_base_asset_volume', 'taker_buy_quote_asset_volume', 'ignore', 'index'], axis=1, errors="ignore")
        # print(hist.head())
        #hist = hist.reset_index() # already reset
        hist['Date'] = hist['Date'].dt.date
        hist = hist.set_index('Date')
        hist = hist.dropna()

        if len(hist) <= 10:
            return hist

        csvPath = "%s/%s.csv" % (self.csvDir, _code)
        print("--------", _code, csvPath)
        hist.to_csv(csvPath)
        return hist

    def getKByList(self):
        # Define the file path
        list_path = "%s/project/data/list/bn.txt" % (self.home)

        try:
            with open(list_path, 'r') as file:
                for line in file:
                    code = line.strip()
                    print(code)
                    data = self.getK(code)
        except FileNotFoundError:
            print(f"File '{list_path}' not found.")
        except Exception as e:
            print(f"An error occurred: {e}")


if __name__=="__main__":
    os.environ['https_proxy'] = 'http://127.0.0.1:7890'
    os.environ['http_proxy'] = 'http://127.0.0.1:7890'
    os.environ['all_proxy'] = 'socks5://127.0.0.1:7890'

    bn = BN()

    data = bn.getK('BTCUSDT')
    print(data)


import os
import akshare as ak
import warnings
warnings.filterwarnings("ignore")

class AKER:
    def __init__(self, _market='cn') -> None:
        self.market = _market

        self.home = "%s" % (os.getenv('HOME'))
        self.csvDir = f'{self.home}/project/data/{self.market}'

    def getK(self, _code):
        try:
            adjust = 'qfq'
            if self.market == 'hk':
                hist = ak.stock_hk_daily(symbol=_code, adjust=adjust)
            else:
                hist = ak.stock_zh_a_hist(symbol=_code, period="daily", adjust=adjust)
        except Exception as e:
            print(f'failed: {e}')
            return None

        #print(hist.head())
        if len(hist) <= 10:
            return hist

        if self.market == 'hk':
            hist.columns = ["Date", "Open", "High",  "Low", "Close", "Volume"]
            hist['Code'] = _code
            hist = hist.reset_index()
            #hist['Date'] = hist['Date'].dt.date
            hist = hist.set_index('Date')
        else:
            hist = hist.drop('振幅', axis=1, errors="ignore")
            hist = hist.drop('涨跌额', axis=1, errors="ignore")
            hist.columns = ["Date", "Code", "Open", "Close",  "High", "Low", "Volume", "Amount", "Incr", "TurnOver"]
            hist = hist.reset_index()
            hist = hist.set_index('Date')
        hist = hist.dropna()
        csvPath = "%s/%s.csv" % (self.csvDir, _code)
        #print("--------", _code, csvPath)
        hist.to_csv(csvPath)
        return hist

    def getKByList(self):
        # Define the file path
        list_path = "%s/project/data/list/cn.txt" % (self.home)
        if self.market == 'hk':
            list_path = "%s/project/data/list/hk.txt" % (self.home)

        try:
            with open(list_path, 'r') as file:
                for line in file:
                    code = line.strip()
                    print(code)
                    _ = self.getK(code)
        except FileNotFoundError:
            print(f"File '{list_path}' not found.")
        except Exception as e:
            print(f"An error occurred: {e}")

if __name__=="__main__":
    h = AKER('hk')
    data = h.getK('00700')
    print(data)

    a = AKER()
    data = a.getK('601088')
    print(data)


#!/usr/bin/env python
# -*- encoding: utf8 -*-

import pandas as pd
from abc import ABC, abstractmethod

import warnings
warnings.filterwarnings("ignore")

from jing.trend import Trend
from jing.y import Y

pd.set_option('display.max_columns', None)

class Rule(ABC):
    @abstractmethod
    def run(self):
        pass

class RuleMa50Ma200(Rule):
    def __init__(self, _code, _ref) -> None:
        self.code = _code
        self.ref  = _ref
        self.name = self.__class__.__name__

    def run(self):
        code = self.code
        ref = self.ref
        dt = ref.date(0)

        print(f"{code} {dt} len: {ref.len()}")

        # Rule1: break up ma50
        if ref.close(0) > ref.ma50(0) and ref.close(1) < ref.ma50(1):
            print(f"[{code}][{dt}] break up ma50, nice")
            pass
        else:
            print(f"[{code}][{dt}] not break up ma50, bye")
            #logger.debug(ref.df.head(5))
            return False

        # Rule2: ma200 ascending many days!
        trend = Trend()
        per, rate = trend.ascendPercentile(ref.Ma200())
        #logger.debug("[%s] ma200: per: %s, rate: %s", code, per, rate)
        if per >= 98 and rate > 20:
            print(f"[{code}][{dt}] ma200 asceding[{per}, {rate}], nice")
            pass
        else:
            print(f"[{code}][{dt}] ma200 not asceding[{per},{rate}], bye")
            return False

        # Rule3: ma50 is always upper than ma200
        per, rate, start = trend.ascendCross(ref.Ma50(), ref.Ma200())
        #logger.debug("[%s] ma50 : per: %s, rate: %s, start: %s", code, per, rate, start)
        if per >= 98 and rate > 25:
            print(f"[{code}][{dt}] ma50 keeps upper[{per}, {rate}], nice")
        else:
            print(f"[{code}][{dt}] ma50 not keeps uppers[{per}, {rate}], bye")
            return False

        # Rule4: ma50 is not far away from ma200
        rate = 100 * (ref.ma50(0) / ref.ma200(0) - 1)
        #logger.debug("[%s] near: rate: %s", code, rate)
        # Rate:
        # - NFLX: 5.7%
        # - AMD: 5.7
        if rate > 2 and rate < 15:
            print(f"[{code}][{dt}] ma50 is near to ma200 [{rate}], nice")
        else:
            print(f"[{code}][{dt}] ma50 is not near to ma200 [{rate}], bye")
            return False

        # Rule5: close-price once is near to ma200
        n = 50
        minRate = trend.approach(ref.Close(), ref.Ma200(), n)
        if minRate < 10 and minRate > -10:
            print(f"[{code}][{dt}] close is near to ma200 [{minRate}], nice")
        else:
            print(f"[{code}][{dt}] close is not near to ma200[{minRate}], bye")
            return False

        print(f"[{code}][{dt}] ruleMa50Ma200 bingo", code, dt)
        return True


class RulePriceBreakout(Rule):
    def __init__(self, _code, _ref) -> None:
        self.code = _code
        self.ref  = _ref
        self.name = self.__class__.__name__

    def run(self):
        code = self.code
        ref = self.ref
        dt = ref.date(0)

        print(f"[{code}][{dt}] len: {ref.len()}")

        # Rule0: rate & vr
        rate = 100 * (ref.close(0) / ref.close(1) - 1)
        vr   = ref.vol(0) / ref.vma50(0)
        if rate >= 9 and vr >= 4:
            print(f"[{code}][{dt}] rate[{rate}], vr[{vr}], nice")
            pass
        else:
            print(f"[{code}][{dt}] rate[{rate}], vr[{vr}] doesn't match, bye")
            return False

        trend = Trend()
        # Rule1: price breakout
        n = 200
        b = trend.breakout(ref.Close(), n)
        if b:
            print(f"[{code}][{dt}] breakout {n} days, nice")
            pass
        else:
            print(f"[{code}][{dt}] not breakout")
            return False

        # Rule2: close is near to ma200
        n = 50
        near = trend.approach(ref.Close(), ref.Ma200(), n)
        if near < 20:
            print(f"[{code}][{dt}] approaching[{near}], nice")
        else:
            print(f"[{code}][{dt}] approaching")
            return False

        # Rule3: ma200 ascending many days!
        trend = Trend()
        per, rate = trend.ascendPercentile(ref.Ma200())
        #logger.debug("[%s] ma200: per: %s, rate: %s", code, per, rate)
        if per >= 99 and rate > 20:
            print(f"[{code}][{dt}] ma200 asceding[{per}, {rate}], nice")
            pass
        else:
            print(f"[{code}][{dt}] ma200 not asceding[{per}, {rate}], bye")
            return False

        # Rule4: ma50 is always upper than ma200
        per, rate, start = trend.ascendCross(ref.Ma50(), ref.Ma200())
        #logger.debug("[%s] ma50 : per: %s, rate: %s, start: %s", code, per, rate, start)
        if per >= 99 and rate > 25:
            print(f"[{code}][{dt}] ma50 keeps upper[{per}, {rate}], nice")
        else:
            print(f"[{code}][{dt}] ma50 not keeps uppers[{per}, {rate}], bye")
            return False

        # Rule5: ma50 is not far away from ma200
        rate = 100 * (ref.ma50(0) / ref.ma200(0) - 1)
        if rate > 2 and rate < 20:
            print(f"[{code}][{dt}] ma50 is near to ma200 [{rate}], nice")
        else:
            print(f"[{code}][{dt}] ma50 is not near to ma200 [{rate}], bye")
            return False

        print(f"[{code}][{dt}] rulePriceBreakout bingo")
        return True


class RuleVolumeBreakout(Rule):
    def __init__(self, _code, _ref) -> None:
        self.code = _code
        self.ref  = _ref
        self.name = self.__class__.__name__

    def run(self):
        code = self.code
        ref = self.ref
        dt = ref.date(0)

        print(f"[{code}][{dt}] len: {ref.len()}")

        # Rule1: rate & vr
        rate = 100 * (ref.close(0) / ref.close(1) - 1)
        vr   = ref.vol(0) / ref.vma50(0)
        if rate >= 30 and vr >= 6:
            print(f"[{code}][{dt}] rate[{rate}], vr[{vr}], nice")
            pass
        else:
            print(f"[{code}][{dt}] rate[{rate}], vr[{vr}] doesn't match, bye")
            return False

        trend = Trend()
        # Rule2: price breakout
        n = 50
        b = trend.breakout(ref.Close(), n)
        if b:
            print(f"[{code}][{dt}] breakout {n} days, nice")
            pass
        else:
            print(f"[{code}][{dt}] not breakout")
            return False

        # Rule3: close was near to ma200
        n = 50
        near = trend.approach(ref.Close(), ref.Ma200(), n)
        if near < 20:
            print(f"[{code}][{dt}] approaching[{near}], nice")
        else:
            print(f"[{code}][{dt}] approaching")
            return False

        # Rule3: ma200 ascending many days!
        trend = Trend()
        per, rate = trend.ascendPercentile(ref.Ma200(), 50)
        #logger.debug("[%s] ma200: per: %s, rate: %s", code, per, rate)
        if per >= 90 and rate > 5:
            print(f"[{code}][{dt}] ma200 asceding[{per}, {rate}], nice")
            pass
        else:
            print(f"[{code}][{dt}] ma200 not asceding[{per}, {rate}], bye")
            return False

        # Rule4: ma50 is always upper than ma200
        per, rate, start = trend.ascendCross(ref.Ma50(), ref.Ma200(), 100, 50)
        #logger.debug("[%s] ma50 : per: %s, rate: %s, start: %s", code, per, rate, start)
        if per >= 99 and rate > 5:
            print(f"[{code}][{dt}] ma50 keeps upper[{per}, {rate}], nice")
        else:
            print(f"[{code}][{dt}] ma50 not keeps uppers[{per}, {rate}], bye")
            return False

        # Rule5: ma50 is not far away from ma200
        rate = 100 * (ref.ma50(0) / ref.ma200(0) - 1)
        if rate > 2 and rate < 20:
            print(f"[{code}][{dt}] ma50 is near to ma200 [{rate}], nice")
        else:
            print(f"[{code}][{dt}] ma50 is not near to ma200 [{rate}], bye")
            return False

        print(f"[{code}][{dt}] ruleVolumeBreakout bingo")
        return True

class RuleSimple(Rule):
    def __init__(self, _code, _ref) -> None:
        self.code = _code
        self.ref  = _ref
        self.name = self.__class__.__name__

    def run(self):
        code = self.code
        ref = self.ref
        dt = ref.date(0)

        print(f"[{code}][{dt}] len: {ref.len()}")

        # Rule1: rate & vr
        rate = 100 * (ref.close(0) / ref.close(1) - 1)
        vr   = ref.vol(0) / ref.vma50(0)
        if rate >= 20 and vr >= 8:
            print(f"[{code}][{dt}] rate[{rate}], vr[{vr}], nice")
            pass
        else:
            print(f"[{code}][{dt}] rate[{rate}], vr[{vr}] doesn't match, bye")
            return

        print(f"[{code}][{dt}] RuleSimple bingo")
        return True


if __name__=="__main__":
    code = "IONQ"
    date = '2024-09-27'

    y = Y("IONQ", _date=date)
    ref = y.ref

    rule = RuleSimple(code, ref)
    rule.run()




#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from jing.data_center import DataCenter
from jing.reference import Reference

class Trend:
    def __init__(self) -> None:
        self.step = 20
        pass

    def ascendPercentile(self, _ma200, _limit=120):
        up = 0
        down = 0
        length = len(_ma200)
        n = 0
        for i, v in _ma200.items():
            # recent X days
            if n >= _limit:
                break

            # incase overflow
            k = n + self.step
            if k >= length:
                break

            if v >= _ma200.iloc[k]:
                up += 1
            else:
                down += 1

            n += 1 

        n = up + down
        percentile = 100.00 * (up / n)
        rate = 100.00 * (_ma200.iloc[0] / _ma200.iloc[n-1] - 1)
        return percentile, rate

    # cross:
    # - ma50, short
    # - ma200, long
    def ascendCross(self, _ma50, _ma200, _cross=200, _upLimit=100):
        up = 0
        down = 0
        length = len(_ma200)
        n = 0
        shortUp = 0
        start = ""
        for i, v in _ma200.items():

            # recent X days
            if n >= _cross:
                break

            if _ma50[i] < _ma200.loc[i]:
                start = i
                break

            if _ma50.loc[i] > _ma200.loc[i]:
                shortUp += 1

            n += 1

        if shortUp < _upLimit:
            print("shortUp", shortUp)
            return 0, 0, '2024-01-01'

        percentile = 100.00 * (shortUp / n)
        rate = 100.00 * (_ma50.iloc[0] / _ma50.iloc[n-1] - 1)
        return percentile, rate, start

    # price breakout recent n days
    def breakout(self, _close, _n):
        this = _close.iloc[0]
        prev  = _close.iloc[1]
        maxPrice = _close[1:_n].max()
        # print("price: ", this)
        # print("prev: ", prev)
        # print("max: ", maxPrice)
        v = this > maxPrice and prev < maxPrice
        return v

    def approach(self, _close, _ma200, _n):
        close = _close[:_n]
        ma200 = _ma200[:_n]
        rate = 100 * (close / ma200 - 1)
        minRate = rate.min()
        # print("min rate: %s", minRate)
        return minRate

if __name__ == "__main__":
    data_center = DataCenter()
    df = data_center.csv("SMCI", '2024-01-19')
    print(df)
    ref = Reference(df)
    ma50 = ref.Ma50()
    ma200 = ref.Ma200()
    close = ref.Close()
    trend = Trend()
    per, rate = trend.ascendPercentile(ma200)
    print("trend", per, rate)
    # model = trend.linear(ma200)
    # print(model.summary())

    per, rate, start = trend.ascendCross(ma50, ma200)
    print("cross", per, rate, start)

    trend.breakout(close, 200)
    trend.approach(close, ma200, 50)
